@@ -177,9 +177,6 @@ gboolean grl_jamendo_plugin_init (GrlRegistry *registry,
 
 static const GList *grl_jamendo_source_supported_keys (GrlSource *source);
 
-static GrlCaps *grl_jamendo_source_get_caps (GrlSource *source,
-                                             GrlSupportedOps operation);
-
 static void grl_jamendo_source_resolve (GrlSource *source,
                                         GrlSourceResolveSpec *rs);
 
@@ -228,6 +225,7 @@ grl_jamendo_source_new (void)
 		       "source-id", SOURCE_ID,
 		       "source-name", SOURCE_NAME,
 		       "source-desc", SOURCE_DESC,
+		       "supported-media", GRL_MEDIA_TYPE_AUDIO,
 		       NULL);
 }
 
@@ -259,7 +257,6 @@ grl_jamendo_source_class_init (GrlJamendoSourceClass * klass)
 
   source_class->cancel = grl_jamendo_source_cancel;
   source_class->supported_keys = grl_jamendo_source_supported_keys;
-  source_class->get_caps = grl_jamendo_source_get_caps;
   source_class->resolve = grl_jamendo_source_resolve;
   source_class->browse = grl_jamendo_source_browse;
   source_class->query = grl_jamendo_source_query;
@@ -1363,15 +1360,3 @@ grl_jamendo_source_cancel (GrlSource *source, guint operation_id)
     xpe->cancelled = TRUE;
   }
 }
-
-static GrlCaps *
-grl_jamendo_source_get_caps (GrlSource *source,
-                             GrlSupportedOps operation)
-{
-  static GrlCaps *caps = NULL;
-
-  if (caps == NULL)
-    caps = grl_caps_new ();
-
-  return caps;
-}
