@@ -35,7 +35,7 @@
 #include "grl-tmdb-request.h"
 
 #define GRL_LOG_DOMAIN_DEFAULT tmdb_log_domain
-GRL_LOG_DOMAIN_STATIC(tmdb_log_domain);
+GRL_LOG_DOMAIN(tmdb_log_domain);
 
 #define PLUGIN_ID   TMDB_PLUGIN_ID
 
@@ -44,22 +44,19 @@ GRL_LOG_DOMAIN_STATIC(tmdb_log_domain);
 #define SOURCE_DESC "A source for movie metadata from themoviedb.org"
 
 #define SHOULD_RESOLVE(key) \
-    g_hash_table_contains (closure->keys, GINT_TO_POINTER ((key)))
+    g_hash_table_contains (closure->keys, GRLKEYID_TO_POINTER ((key)))
+#define SHOULD_RESOLVE_SLOW(key) \
+    (closure->slow && SHOULD_RESOLVE (key))
 
 enum {
   PROP_0,
   PROP_API_KEY
 };
 
-static GrlKeyID GRL_TMDB_METADATA_KEY_BACKDROPS = GRL_METADATA_KEY_INVALID;
-static GrlKeyID GRL_TMDB_METADATA_KEY_POSTERS = GRL_METADATA_KEY_INVALID;
+static GrlKeyID GRL_TMDB_METADATA_KEY_BACKDROP = GRL_METADATA_KEY_INVALID;
+static GrlKeyID GRL_TMDB_METADATA_KEY_POSTER = GRL_METADATA_KEY_INVALID;
 static GrlKeyID GRL_TMDB_METADATA_KEY_IMDB_ID = GRL_METADATA_KEY_INVALID;
-static GrlKeyID GRL_TMDB_METADATA_KEY_KEYWORDS = GRL_METADATA_KEY_INVALID;
-static GrlKeyID GRL_TMDB_METADATA_KEY_PERFORMER = GRL_METADATA_KEY_INVALID;
-static GrlKeyID GRL_TMDB_METADATA_KEY_PRODUCER = GRL_METADATA_KEY_INVALID;
-static GrlKeyID GRL_TMDB_METADATA_KEY_DIRECTOR = GRL_METADATA_KEY_INVALID;
-static GrlKeyID GRL_TMDB_METADATA_KEY_AGE_CERTIFICATES = GRL_METADATA_KEY_INVALID;
-static GrlKeyID GRL_TMDB_METADATA_KEY_ORIGINAL_TITLE = GRL_METADATA_KEY_INVALID;
+static GrlKeyID GRL_TMDB_METADATA_KEY_TMDB_ID = GRL_METADATA_KEY_INVALID;
 
 struct _GrlTmdbSourcePrivate {
   char *api_key;
@@ -144,13 +141,13 @@ grl_tmdb_source_plugin_init (GrlRegistry *registry,
     return FALSE;
   }
 
-  GRL_TMDB_METADATA_KEY_BACKDROPS =
+  GRL_TMDB_METADATA_KEY_BACKDROP =
     register_metadata_key (registry,
-                           "tmdb-backdrops",
-                           "tmdb-backdrops",
+                           "tmdb-backdrop",
+                           "tmdb-backdrop",
                            "A list of URLs for movie backdrops");
 
-  GRL_TMDB_METADATA_KEY_POSTERS =
+  GRL_TMDB_METADATA_KEY_POSTER =
     register_metadata_key (registry,
                            "tmdb-poster",
                            "tmdb-poster",
@@ -162,41 +159,11 @@ grl_tmdb_source_plugin_init (GrlRegistry *registry,
                            "tmdb-imdb-id",
                            "ID of this movie at imdb.org");
 
-  GRL_TMDB_METADATA_KEY_KEYWORDS =
+  GRL_TMDB_METADATA_KEY_TMDB_ID =
     register_metadata_key (registry,
-                           "tmdb-keywords",
-                           "tmdb-keywords",
-                           "A list of keywords about the movie");
-
-  GRL_TMDB_METADATA_KEY_PERFORMER =
-    register_metadata_key (registry,
-                           "tmdb-performer",
-                           "tmdb-performer",
-                           "A list of actors taking part in the movie");
-
-  GRL_TMDB_METADATA_KEY_PRODUCER =
-    register_metadata_key (registry,
-                           "tmdb-producer",
-                           "tmdb-producer",
-                           "A list of producers of the movie");
-
-  GRL_TMDB_METADATA_KEY_DIRECTOR =
-    register_metadata_key (registry,
-                           "tmdb-director",
-                           "tmdb-director",
-                           "Director of the movie");
-
-  GRL_TMDB_METADATA_KEY_AGE_CERTIFICATES =
-    register_metadata_key (registry,
-                           "tmdb-age-certificates",
-                           "tmdb-age-certificates",
-                           "Semicolon-separated list of all age certificates");
-
-  GRL_TMDB_METADATA_KEY_ORIGINAL_TITLE =
-    register_metadata_key (registry,
-                           "tmdb-original-title",
-                           "tmdb-original-title",
-                           "Original title of a movie");
+                           "tmdb-id",
+                           "tmdb-id",
+                           "ID of this movie at tmdb.org");
 
   GrlTmdbSource *source = grl_tmdb_source_new (api_key);
   grl_registry_register_source (registry,
@@ -263,39 +230,43 @@ grl_tmdb_source_init (GrlTmdbSource *self)
 
   /* Fast keys */
   g_hash_table_add (self->priv->supported_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_BACKDROPS));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_THUMBNAIL));
   g_hash_table_add (self->priv->supported_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_ORIGINAL_TITLE));
+                    GRLKEYID_TO_POINTER (GRL_TMDB_METADATA_KEY_BACKDROP));
   g_hash_table_add (self->priv->supported_keys,
-                    GINT_TO_POINTER (GRL_METADATA_KEY_RATING));
+                    GRLKEYID_TO_POINTER (GRL_TMDB_METADATA_KEY_POSTER));
   g_hash_table_add (self->priv->supported_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_POSTERS));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_ORIGINAL_TITLE));
   g_hash_table_add (self->priv->supported_keys,
-                    GINT_TO_POINTER (GRL_METADATA_KEY_PUBLICATION_DATE));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_RATING));
+  g_hash_table_add (self->priv->supported_keys,
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_PUBLICATION_DATE));
+  g_hash_table_add (self->priv->supported_keys,
+                    GRLKEYID_TO_POINTER (GRL_TMDB_METADATA_KEY_TMDB_ID));
 
   /* Slow keys */
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_METADATA_KEY_SITE));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_SITE));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_METADATA_KEY_GENRE));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_GENRE));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_METADATA_KEY_STUDIO));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_STUDIO));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_METADATA_KEY_DESCRIPTION));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_DESCRIPTION));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_METADATA_KEY_CERTIFICATE));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_CERTIFICATE));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_IMDB_ID));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_REGION));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_KEYWORDS));
+                    GRLKEYID_TO_POINTER (GRL_TMDB_METADATA_KEY_IMDB_ID));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_PERFORMER));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_KEYWORD));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_PRODUCER));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_PERFORMER));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_DIRECTOR));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_PRODUCER));
   g_hash_table_add (self->priv->slow_keys,
-                    GINT_TO_POINTER (GRL_TMDB_METADATA_KEY_AGE_CERTIFICATES));
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_DIRECTOR));
 
   self->priv->wc = grl_net_wc_new ();
   grl_net_wc_set_throttling (self->priv->wc, 1);
@@ -404,15 +375,14 @@ register_metadata_key (GrlRegistry *registry,
 
 static void
 resolve_closure_callback (ResolveClosure *closure,
-                          GError *outer_error)
+                          const GError *outer_error)
 {
   GError *error = NULL;
 
   if (outer_error && outer_error->domain != GRL_CORE_ERROR) {
     error = g_error_new_literal (GRL_CORE_ERROR,
                                  GRL_CORE_ERROR_RESOLVE_FAILED,
-                                 error->message);
-    g_error_free (outer_error);
+                                 outer_error->message);
   }
 
   closure->rs->callback (GRL_SOURCE (closure->self),
@@ -473,43 +443,18 @@ director_filter (JsonNode *element)
   return g_strdup (json_object_get_string_member (object, "name"));
 }
 
-static char *
-us_release_filter (JsonNode *node)
+static GDateTime *
+parse_date (const gchar *iso8601_string)
 {
-  JsonObject *object;
-  const char *country;
+  char *padded_date;
+  GDateTime *date;
+  GTimeVal tv;
 
-  if (!JSON_NODE_HOLDS_OBJECT (node)) {
-    return NULL;
-  }
-
-  object = json_node_get_object (node);
-  country = json_object_get_string_member (object, "iso_3166_1");
-  if (g_ascii_strcasecmp (country, "US") == 0) {
-    return g_strdup (json_object_get_string_member (object, "certification"));
-  }
-
-  return NULL;
-}
-
-static char *
-all_releases_filter (JsonNode *node)
-{
-  JsonObject *object;
-  const char *country, *cert;
-
-  if (!JSON_NODE_HOLDS_OBJECT (node)) {
-    return NULL;
-  }
-
-  object = json_node_get_object (node);
-  country = json_object_get_string_member (object, "iso_3166_1");
-  cert = json_object_get_string_member (object, "certification");
-  if (cert == NULL || strlen (cert) == 0) {
-    /* This is only a release date, no age cert */
-    return NULL;
-  }
-  return g_strconcat (country, ":", cert, NULL);
+  padded_date = g_strconcat (iso8601_string, "T00:00:00Z", NULL);
+  g_time_val_from_iso8601 (padded_date, &tv);
+  date = g_date_time_new_from_timeval_utc (&tv);
+  g_free (padded_date);
+  return date;
 }
 
 static char *
@@ -533,9 +478,29 @@ neutral_backdrop_filter (JsonNode *node)
   return NULL;
 }
 
-static void on_request_ready (GObject *source,
-                              GAsyncResult *result,
-                              gpointer user_data) {
+static void
+add_image (GrlTmdbSource *self,
+           GrlMedia      *media,
+           GrlKeyID       detail_key,
+           const char    *image_path)
+{
+    SoupURI *uri;
+    char *str;
+
+    str = g_strconcat ("original", image_path, NULL);
+    uri = soup_uri_new_with_base (self->priv->image_base_uri, str);
+    g_free (str);
+
+    str = soup_uri_to_string (uri, FALSE);
+    grl_media_add_thumbnail (media, str);
+    grl_data_add_string (GRL_DATA (media), detail_key, str);
+    g_free (str);
+}
+
+static void
+on_request_ready (GObject *source,
+                  GAsyncResult *result,
+                  gpointer user_data) {
   ResolveClosure *closure = (ResolveClosure *) user_data;
   GrlTmdbRequest *request = GRL_TMDB_REQUEST (source);
   GError *error = NULL;
@@ -560,8 +525,7 @@ static void on_request_ready (GObject *source,
         iter = values = grl_tmdb_request_get_string_list (request, "$.genres..name");
         while (iter != NULL) {
           grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_METADATA_KEY_GENRE,
-                               (char *) iter->data);
+                               GRL_METADATA_KEY_GENRE, iter->data);
           iter = iter->next;
         }
         g_list_free_full (values, g_free);
@@ -571,8 +535,7 @@ static void on_request_ready (GObject *source,
         iter = values = grl_tmdb_request_get_string_list (request, "$.production_companies..name");
         while (iter != NULL) {
           grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_METADATA_KEY_STUDIO,
-                               (char *) iter->data);
+                               GRL_METADATA_KEY_STUDIO, iter->data);
           iter = iter->next;
         }
         g_list_free_full (values, g_free);
@@ -608,50 +571,31 @@ static void on_request_ready (GObject *source,
     break;
     case GRL_TMDB_REQUEST_DETAIL_MOVIE_IMAGES:
     {
-      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROPS)) {
+      /* Add posters first and backdrops later.
+       * Posters more likely make a good thumbnail than backdrops.
+       */
+      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTER)) {
         iter = values = grl_tmdb_request_get_string_list_with_filter (request,
-                                                                      "$.backdrops",
+                                                                      "$.posters",
                                                                       neutral_backdrop_filter);
         while (iter != NULL) {
-          SoupURI *backdrop_uri;
-          char *path;
-
-          path = g_strconcat ("original", (char *) iter->data, NULL);
-
-          backdrop_uri = soup_uri_new_with_base (closure->self->priv->image_base_uri,
-                                                 path);
-          g_free (path);
-          path = soup_uri_to_string (backdrop_uri, FALSE);
-
-          grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_TMDB_METADATA_KEY_BACKDROPS,
-                               path);
-          g_free (path);
+          add_image (closure->self, closure->rs->media,
+                     GRL_TMDB_METADATA_KEY_POSTER,
+                     iter->data);
 
           iter = iter->next;
         }
         g_list_free_full (values, g_free);
       }
 
-      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTERS)) {
+      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROP)) {
         iter = values = grl_tmdb_request_get_string_list_with_filter (request,
-                                                                      "$.posters",
+                                                                      "$.backdrops",
                                                                       neutral_backdrop_filter);
         while (iter != NULL) {
-          SoupURI *backdrop_uri;
-          char *path;
-
-          path = g_strconcat ("original", (char *) iter->data, NULL);
-
-          backdrop_uri = soup_uri_new_with_base (closure->self->priv->image_base_uri,
-                                                 path);
-          g_free (path);
-          path = soup_uri_to_string (backdrop_uri, FALSE);
-
-          grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_TMDB_METADATA_KEY_POSTERS,
-                               path);
-          g_free (path);
+          add_image (closure->self, closure->rs->media,
+                     GRL_TMDB_METADATA_KEY_BACKDROP,
+                     iter->data);
 
           iter = iter->next;
         }
@@ -661,13 +605,11 @@ static void on_request_ready (GObject *source,
     break;
     case GRL_TMDB_REQUEST_DETAIL_MOVIE_KEYWORDS:
     {
-      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_KEYWORDS)) {
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_KEYWORD)) {
         iter = values = grl_tmdb_request_get_string_list (request,
                                                           "$.keywords..name");
         while (iter != NULL) {
-          grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_TMDB_METADATA_KEY_KEYWORDS,
-                               (char *) iter->data);
+          grl_media_add_keyword (closure->rs->media, iter->data);
           iter = iter->next;
         }
         g_list_free_full (values, g_free);
@@ -676,41 +618,38 @@ static void on_request_ready (GObject *source,
     break;
     case GRL_TMDB_REQUEST_DETAIL_MOVIE_CAST:
     {
-      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_PERFORMER)) {
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_PERFORMER)) {
         values = grl_tmdb_request_get_string_list (request, "$.cast..name");
         iter = values;
         while (iter != NULL) {
-          grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_TMDB_METADATA_KEY_PERFORMER,
-                               (char *) iter->data);
+          grl_media_video_add_performer (GRL_MEDIA_VIDEO (closure->rs->media),
+                                         iter->data);
           iter = iter->next;
         }
         g_list_free_full (values, g_free);
       }
 
-      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_PRODUCER)) {
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_PRODUCER)) {
         values = grl_tmdb_request_get_string_list_with_filter (request,
                                                                "$.crew[*]",
                                                                producer_filter);
         iter = values;
         while (iter != NULL) {
-          grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_TMDB_METADATA_KEY_PRODUCER,
-                               (char *) iter->data);
+            grl_media_video_add_producer (GRL_MEDIA_VIDEO (closure->rs->media),
+                                          iter->data);
           iter = iter->next;
         }
         g_list_free_full (values, g_free);
       }
 
-      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_DIRECTOR)) {
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_DIRECTOR)) {
         values = grl_tmdb_request_get_string_list_with_filter (request,
                                                                "$.crew[*]",
                                                                director_filter);
         iter = values;
         while (iter != NULL) {
-          grl_data_add_string (GRL_DATA (closure->rs->media),
-                               GRL_TMDB_METADATA_KEY_DIRECTOR,
-                               (char *) iter->data);
+          grl_media_video_add_director (GRL_MEDIA_VIDEO (closure->rs->media),
+                                        iter->data);
           iter = iter->next;
         }
         g_list_free_full (values, g_free);
@@ -719,38 +658,30 @@ static void on_request_ready (GObject *source,
     break;
     case GRL_TMDB_REQUEST_DETAIL_MOVIE_RELEASE_INFO:
     {
-      GString *string;
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_REGION) ||
+              SHOULD_RESOLVE (GRL_METADATA_KEY_CERTIFICATE) ||
+              SHOULD_RESOLVE_SLOW (GRL_METADATA_KEY_PUBLICATION_DATE)) {
+        values = grl_tmdb_request_get_list_with_filter (request,
+                                                        "$.countries[*]",
+                                                        NULL);
 
-      if (SHOULD_RESOLVE (GRL_METADATA_KEY_CERTIFICATE)) {
-        values = grl_tmdb_request_get_string_list_with_filter (request,
-                                                               "$.countries[*]",
-                                                               us_release_filter);
-        if (values != NULL) {
-          grl_media_set_certificate (closure->rs->media,
-                                     (char *) values->data);
-          g_list_free_full (values, g_free);
-        }
-      }
+        for (iter = values; iter != NULL; iter = iter->next) {
+          const char *region, *cert, *date;
+          GDateTime *pubdate;
+          JsonObject *object;
 
-      if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_AGE_CERTIFICATES)) {
-        values = grl_tmdb_request_get_string_list_with_filter (request,
-                                                               "$.countries[*]",
-                                                               all_releases_filter);
-        iter = values;
-        string = g_string_new (NULL);
-        while (iter != NULL) {
-          g_string_append (string, (char *) iter->data);
-          iter = iter->next;
-          if (iter != NULL) {
-            g_string_append_c (string, ';');
-          }
+          object = json_node_get_object (iter->data);
+          region = json_object_get_string_member (object, "iso_3166_1");
+          cert = json_object_get_string_member (object, "certification");
+          date = json_object_get_string_member (object, "release_date");
+          pubdate = parse_date (date);
+
+          grl_media_add_region_data (closure->rs->media, region, pubdate, cert);
+
+          g_date_time_unref (pubdate);
         }
-        if (string->str != NULL) {
-          grl_data_set_string (GRL_DATA (closure->rs->media),
-                               GRL_TMDB_METADATA_KEY_AGE_CERTIFICATES,
-                               string->str);
-        }
-        g_string_free (string, TRUE);
+
+        g_list_free (values);
       }
     }
     break;
@@ -800,7 +731,6 @@ on_search_ready (GObject *source,
   GrlTmdbSource *self = closure->self;
   GValue *value;
   GError *error = NULL;
-  char *padded_date;
 
   GRL_DEBUG ("Initial search ready...");
   if (!grl_tmdb_request_run_finish (GRL_TMDB_REQUEST (source),
@@ -808,6 +738,7 @@ on_search_ready (GObject *source,
                                     &error)) {
     resolve_closure_callback (closure, error);
     resolve_closure_free (closure);
+    g_error_free (error);
     return;
   }
 
@@ -827,24 +758,27 @@ on_search_ready (GObject *source,
                                  "Remote data did not contain valid ID");
     resolve_closure_callback (closure, error);
     resolve_closure_free (closure);
+    g_error_free (error);
     return;
   }
+
+  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_TMDB_ID)) {
+    char *tmdb_id = g_strdup_printf ("%" G_GINT64_FORMAT,
+                                     g_value_get_int64 (value));
+    grl_data_set_string (GRL_DATA (closure->rs->media),
+                         GRL_TMDB_METADATA_KEY_TMDB_ID, tmdb_id);
+    g_free (tmdb_id);
+  }
+
   closure->id = g_value_get_int64 (value);
   g_value_unset (value);
 
   if (SHOULD_RESOLVE (GRL_METADATA_KEY_PUBLICATION_DATE)) {
     value = grl_tmdb_request_get (request, "$.results[0].release_date");
     if (value != NULL) {
-      GDateTime *pubdate;
-      GTimeVal tv;
-      padded_date = g_strconcat (g_value_get_string (value), "T00:00:00Z", NULL);
-      g_time_val_from_iso8601 (padded_date, &tv);
-      pubdate = g_date_time_new_from_timeval_utc (&tv);
-
-      grl_media_set_publication_date (closure->rs->media,
-                                      pubdate);
+      GDateTime *pubdate = parse_date (g_value_get_string (value));
+      grl_media_set_publication_date (closure->rs->media, pubdate);
       g_date_time_unref (pubdate);
-      g_free (padded_date);
       g_value_unset (value);
     }
   }
@@ -859,53 +793,37 @@ on_search_ready (GObject *source,
     }
   }
 
-  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROPS)) {
+  /* Add posters first and backdrops later.
+   * Posters more likely make a good thumbnail than backdrops.
+   */
+  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTER)) {
+    value = grl_tmdb_request_get (request, "$.results[0].poster_path");
+    if (value != NULL) {
+        add_image (closure->self, closure->rs->media,
+                   GRL_TMDB_METADATA_KEY_POSTER,
+                   g_value_get_string (value));
+
+        g_value_unset (value);
+
+    }
+  }
+
+  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROP)) {
     value = grl_tmdb_request_get (request, "$.results[0].backdrop_path");
     if (value != NULL) {
-      SoupURI *backdrop_uri;
-      char *path;
+      add_image (closure->self, closure->rs->media,
+                 GRL_TMDB_METADATA_KEY_BACKDROP,
+                 g_value_get_string (value));
 
-      path = g_strconcat ("original", g_value_get_string (value), NULL);
-
-      backdrop_uri = soup_uri_new_with_base (closure->self->priv->image_base_uri,
-                                             path);
-      g_free (path);
-      path = soup_uri_to_string (backdrop_uri, FALSE);
-
-      grl_data_add_string (GRL_DATA (closure->rs->media),
-                           GRL_TMDB_METADATA_KEY_BACKDROPS,
-                           path);
-      g_free (path);
       g_value_unset (value);
     }
   }
 
-  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTERS)) {
-    value = grl_tmdb_request_get (request, "$.results[0].poster_path");
+  if (SHOULD_RESOLVE (GRL_METADATA_KEY_CERTIFICATE)) {
+    value = grl_tmdb_request_get (request, "$.results[0].adult");
     if (value != NULL) {
-      SoupURI *backdrop_uri;
-      char *path;
-
-      path = g_strconcat ("original", g_value_get_string (value), NULL);
-
-      backdrop_uri = soup_uri_new_with_base (closure->self->priv->image_base_uri,
-                                             path);
-      g_free (path);
-      path = soup_uri_to_string (backdrop_uri, FALSE);
-
-      grl_data_add_string (GRL_DATA (closure->rs->media),
-                           GRL_TMDB_METADATA_KEY_POSTERS,
-                           path);
-      g_free (path);
-    }
-  }
-
-  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_ORIGINAL_TITLE)) {
-    value = grl_tmdb_request_get (request, "$.results[0].original_title");
-    if (value != NULL) {
-      grl_data_set_string (GRL_DATA (closure->rs->media),
-                           GRL_TMDB_METADATA_KEY_ORIGINAL_TITLE,
-                           g_value_get_string (value));
+      if (g_value_get_boolean (value))
+        grl_media_set_certificate (closure->rs->media, "adult");
       g_value_unset (value);
     }
   }
@@ -922,8 +840,8 @@ on_search_ready (GObject *source,
 
   /* We need to do additional requests. Check if we should have resolved
    * images and try to get some more */
-  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROPS) ||
-      SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTERS))
+  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROP) ||
+      SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTER))
     g_queue_push_tail (closure->pending_requests,
                        create_and_run_request (self,
                                                closure,
@@ -939,22 +857,23 @@ on_search_ready (GObject *source,
                                                closure,
                                                GRL_TMDB_REQUEST_DETAIL_MOVIE));
 
-  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_KEYWORDS))
+  if (SHOULD_RESOLVE (GRL_METADATA_KEY_KEYWORD))
     g_queue_push_tail (closure->pending_requests,
                        create_and_run_request (self,
                                                closure,
                                                GRL_TMDB_REQUEST_DETAIL_MOVIE_KEYWORDS));
 
-  if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_PERFORMER) ||
-      SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_PRODUCER) ||
-      SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_DIRECTOR))
+  if (SHOULD_RESOLVE (GRL_METADATA_KEY_PERFORMER) ||
+      SHOULD_RESOLVE (GRL_METADATA_KEY_PRODUCER) ||
+      SHOULD_RESOLVE (GRL_METADATA_KEY_DIRECTOR))
     g_queue_push_tail (closure->pending_requests,
                        create_and_run_request (self,
                                                closure,
                                                GRL_TMDB_REQUEST_DETAIL_MOVIE_CAST));
 
-  if (SHOULD_RESOLVE (GRL_METADATA_KEY_CERTIFICATE) ||
-      SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_AGE_CERTIFICATES))
+  if (SHOULD_RESOLVE (GRL_METADATA_KEY_REGION) ||
+          SHOULD_RESOLVE (GRL_METADATA_KEY_CERTIFICATE) ||
+          SHOULD_RESOLVE_SLOW (GRL_METADATA_KEY_PUBLICATION_DATE))
     g_queue_push_tail (closure->pending_requests,
                        create_and_run_request (self,
                                                closure,
@@ -987,6 +906,8 @@ on_configuration_ready (GObject *source,
       resolve_closure_callback (pending_closure, error);
       resolve_closure_free (pending_closure);
     }
+
+    g_error_free (error);
     return;
   }
 
@@ -1058,9 +979,9 @@ grl_tmdb_source_may_resolve (GrlSource *source,
   GrlTmdbSource *self = GRL_TMDB_SOURCE (source);
 
   if (!g_hash_table_contains (self->priv->supported_keys,
-                             GINT_TO_POINTER (key_id)) &&
+                             GRLKEYID_TO_POINTER (key_id)) &&
       !g_hash_table_contains (self->priv->slow_keys,
-                             GINT_TO_POINTER (key_id)))
+                             GRLKEYID_TO_POINTER (key_id)))
     return FALSE;
 
   /* We can only entertain videos */
@@ -1125,11 +1046,15 @@ grl_tmdb_source_resolve (GrlSource *source,
   /* Copy keys to list for faster lookup */
   while (it) {
     g_hash_table_add (closure->keys, it->data);
+
+    /* Enable slow resolution if slow keys are requested */
     closure->slow |= g_hash_table_contains (self->priv->slow_keys,
                                             it->data);
     it = it->next;
   }
 
+  /* Disable slow resolution if slow keys where requested, but the operation
+   * options explicitly ask for fast resolving only. */
   if (grl_operation_options_get_flags (rs->options) & GRL_RESOLVE_FAST_ONLY)
     closure->slow = FALSE;
 
