@@ -177,7 +177,8 @@ struct _GrlYoutubeSourcePriv {
 #define YOUTUBE_CLIENT_ID "grilo"
 
 static GrlYoutubeSource *grl_youtube_source_new (const gchar *api_key,
-						 const gchar *client_id);
+						 const gchar *client_id,
+						 const gchar *format);
 
 static void grl_youtube_source_set_property (GObject *object,
                                              guint propid,
@@ -193,9 +194,6 @@ static const GList *grl_youtube_source_supported_keys (GrlSource *source);
 
 static const GList *grl_youtube_source_slow_keys (GrlSource *source);
 
-static GrlCaps * grl_youtube_source_get_caps (GrlSource *source,
-                                              GrlSupportedOps operation);
-
 static void grl_youtube_source_search (GrlSource *source,
                                        GrlSourceSearchSpec *ss);
 
@@ -251,6 +249,7 @@ grl_youtube_plugin_init (GrlRegistry *registry,
                          GList *configs)
 {
   gchar *api_key;
+  gchar *format;
   GrlConfig *config;
   gint config_count;
   GrlYoutubeSource *source;
@@ -275,6 +274,7 @@ grl_youtube_plugin_init (GrlRegistry *registry,
     GRL_INFO ("Missing API Key, cannot load plugin");
     return FALSE;
   }
+  format = grl_config_get_string (config, "format");
 
 #if !GLIB_CHECK_VERSION(2,32,0)
   /* libgdata needs this */
@@ -283,7 +283,7 @@ grl_youtube_plugin_init (GrlRegistry *registry,
   }
 #endif
 
-  source = grl_youtube_source_new (api_key, YOUTUBE_CLIENT_ID);
+  source = grl_youtube_source_new (api_key, YOUTUBE_CLIENT_ID, format);
 
   grl_registry_register_source (registry,
                                 plugin,
@@ -291,6 +291,7 @@ grl_youtube_plugin_init (GrlRegistry *registry,
                                 NULL);
 
   g_free (api_key);
+  g_free (format);
 
   return TRUE;
 }
@@ -304,7 +305,7 @@ GRL_PLUGIN_REGISTER (grl_youtube_plugin_init,
 G_DEFINE_TYPE (GrlYoutubeSource, grl_youtube_source, GRL_TYPE_SOURCE);
 
 static GrlYoutubeSource *
-grl_youtube_source_new (const gchar *api_key, const gchar *client_id)
+grl_youtube_source_new (const gchar *api_key, const gchar *client_id, const gchar *format)
 {
   GrlYoutubeSource *source;
   GDataYouTubeService *service;
@@ -330,13 +331,17 @@ grl_youtube_source_new (const gchar *api_key, const gchar *client_id)
 					     "auto-split-threshold",
 					     YOUTUBE_MAX_CHUNK,
                                              "yt-service", service,
+                                             "supported-media", GRL_MEDIA_TYPE_VIDEO,
 					     NULL));
 
   /* Set up quvi */
   if (quvi_init (&(source->priv->quvi_handle)) != QUVI_OK) {
     source->priv->quvi_handle = NULL;
   } else {
-    quvi_setopt (source->priv->quvi_handle, QUVIOPT_FORMAT, "mp4_360p");
+    if (format)
+      quvi_setopt (source->priv->quvi_handle, QUVIOPT_FORMAT, format);
+    else
+      quvi_setopt (source->priv->quvi_handle, QUVIOPT_FORMAT, "mp4_360p");
     quvi_setopt (source->priv->quvi_handle, QUVIOPT_NOVERIFY);
   }
 
@@ -356,7 +361,6 @@ grl_youtube_source_class_init (GrlYoutubeSourceClass * klass)
 
   source_class->supported_keys = grl_youtube_source_supported_keys;
   source_class->slow_keys = grl_youtube_source_slow_keys;
-  source_class->get_caps = grl_youtube_source_get_caps;
   source_class->cancel = grl_youtube_source_cancel;
 
   source_class->search = grl_youtube_source_search;
@@ -1604,15 +1608,3 @@ grl_youtube_source_cancel (GrlSource *source,
     g_cancellable_cancel (cancellable);
   }
 }
-
-static GrlCaps *
-grl_youtube_source_get_caps (GrlSource *source,
-                             GrlSupportedOps operation)
-{
-  static GrlCaps *caps = NULL;
-
-  if (caps == NULL)
-    caps = grl_caps_new ();
-
-  return caps;
-}
